@@ -20,6 +20,7 @@
 #include "mainwindow.h"
 
 #include <KAboutData>
+#include <KCrash>
 #include <KLocalizedString>
 
 #include <QApplication>
@@ -33,20 +34,22 @@ int main (int argc, char **argv)
 
     QApplication app {argc, argv};
 
-    KAboutData aboutData {
+    KCrash::initialize();
+
+    auto aboutData = KAboutData {
         QStringLiteral("kronometer"),   // componentName
-        i18n("Kronometer"), // displayName
-        QStringLiteral("2.0.2"),    // version
+        i18nc("KAboutData display name", "Kronometer"),
+        QStringLiteral("2.1.0"),    // version
         i18n("Kronometer is a simple stopwatch application"), // shortDescription
         KAboutLicense::GPL_V2,    // licenseType
-        i18n("Copyright (C) 2014 Elvis Angelaccio"),    // copyrightStatement
-        QString(),  // otherText
+        i18n("Copyright (C) 2014-2016 Elvis Angelaccio"),    // copyrightStatement
+        {},  // otherText
         QStringLiteral("http://aelog.org/kronometer")   // homePageAddress
     };
 
     aboutData.addAuthor(
         i18n("Elvis Angelaccio"),
-        i18n("Developer"),
+        i18n("Maintainer"),
         QStringLiteral("elvis.angelaccio@kdemail.net"),
         QStringLiteral("http://aelog.org")
     );
@@ -66,13 +69,13 @@ int main (int argc, char **argv)
     app.setWindowIcon(QIcon::fromTheme(QStringLiteral("kronometer")));
 
     // Make sure that the local data directory is available.
-    QFileInfo appdata {QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation)};
+    auto appdata = QFileInfo {QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation)};
     if (not appdata.exists()) {
-        QDir dir {appdata.absolutePath()};
+        auto dir = QDir {appdata.absolutePath()};
         dir.mkdir(appdata.fileName());
     }
 
-    auto window = new MainWindow();
+    auto window = new MainWindow {};
     window->show();
 
     return app.exec();
