@@ -32,6 +32,8 @@
 #include <KConfigDialog>
 #include <KLocalizedString>
 #include <KMessageBox>
+#include <KToggleAction>
+#include <KToolBar>
 
 #include <QAction>
 #include <QApplication>
@@ -42,32 +44,36 @@
 #include <QJsonArray>
 #include <QJsonDocument>
 #include <QJsonObject>
+#include <QMenuBar>
 #include <QPointer>
 #include <QSaveFile>
 #include <QSortFilterProxyModel>
 #include <QSplitter>
 #include <QStatusBar>
 #include <QTableView>
+#include <QTimer>
+#include <QToolButton>
 
 MainWindow::MainWindow(QWidget *parent, const Session& session) : KXmlGuiWindow(parent),
+    m_controlMenuButton {nullptr},
     m_session {session}
 {
-    m_stopwatch = new Stopwatch(this);
-    m_stopwatchDisplay = new TimeDisplay(this);
+    m_stopwatch = new Stopwatch {this};
+    m_stopwatchDisplay = new TimeDisplay {this};
     connect(m_stopwatch, &Stopwatch::time, m_stopwatchDisplay, &TimeDisplay::slotTime);  // bind stopwatch to its display
 
-    m_sessionModel = new SessionModel(this);
+    m_sessionModel = new SessionModel {this};
 
     setupCentralWidget();
-    setupStatusBar();
     setupActions();
-    setupGUI(KXmlGuiWindow::Default, QStringLiteral("kronometerui.rc"));
+    setupGUI(ToolBar | Keys | Save | Create, QStringLiteral("kronometerui.rc"));
 
     // TODO: find a better fix for #351746
     resize(minimumSizeHint());
 
     slotInactive();    // inactive state is the default
     loadSettings();
+    statusBar()->hide();
 
     if (not m_session.isEmpty()) {
         loadSession();
@@ -110,7 +116,7 @@ bool MainWindow::queryClose()
         auto buttonCode = KMessageBox::warningContinueCancel(
                     this,
                     i18n("Do you want to quit and lose your unsaved times?"),
-                    i18n("Confirm quit"),
+                    i18nc("@title:window", "Confirm Quit"),
                     KStandardGuiItem::quit(),
                     KStandardGuiItem::cancel(),
                     QStringLiteral("quit-and-lose-times"));
@@ -139,8 +145,6 @@ bool MainWindow::queryClose()
 
 void MainWindow::slotRunning()
 {
-    m_statusLabel->setText(i18n("Running..."));
-
     m_session.setIsOutdated(true);
     setWindowModified(true);
 
@@ -149,8 +153,7 @@ void MainWindow::slotRunning()
 
 void MainWindow::slotPaused()
 {
-    m_startAction->setText(i18n("Re&sume"));
-    m_statusLabel->setText(i18n("Paused"));
+    m_startAction->setText(i18nc("@action", "Re&sume"));
 
     if (not m_session.isEmpty()) {
         stateChanged(QStringLiteral("pausedSession"));
@@ -168,8 +171,7 @@ void MainWindow::slotPaused()
 
 void MainWindow::slotInactive()
 {
-    m_startAction->setText(i18n("&Start"));
-    m_statusLabel->setText(i18n("Inactive"));
+    m_startAction->setText(i18nc("@action", "&Start"));
 
     m_session.setIsOutdated(false);
 
@@ -190,19 +192,19 @@ void MainWindow::slotPrepareForSleep(bool beforeSleep)
 
 void MainWindow::slotShowSettings()
 {
-    if (KConfigDialog::showDialog(QLatin1String("settings"))) {
+    if (KConfigDialog::showDialog(QStringLiteral("settings"))) {
         return;
     }
 
-    auto dialog = new KConfigDialog(this, QLatin1String("settings"), KronometerConfig::self());
+    auto dialog = new KConfigDialog {this, QStringLiteral("settings"), KronometerConfig::self()};
 
-    auto generalPage = dialog->addPage(new GeneralSettings(this), i18n("General settings"));
+    auto generalPage = dialog->addPage(new GeneralSettings {this}, i18nc("@title:tab", "General Settings"));
     generalPage->setIcon(QIcon::fromTheme(QApplication::windowIcon().name()));
 
-    auto fontPage = dialog->addPage(new FontSettings(this), i18n("Font settings"));
+    auto fontPage = dialog->addPage(new FontSettings {this}, i18nc("@title:tab", "Font Settings"));
     fontPage->setIcon(QIcon::fromTheme(QStringLiteral("preferences-desktop-font")));
 
-    auto colorPage = dialog->addPage(new ColorSettings(this), i18n("Color settings"));
+    auto colorPage = dialog->addPage(new ColorSettings {this}, i18nc("@title:tab", "Color Settings"));
     colorPage->setIcon(QIcon::fromTheme(QStringLiteral("fill-color")));
 
     connect(dialog, &KConfigDialog::settingsChanged, this, &MainWindow::slotWriteSettings);
@@ -214,7 +216,7 @@ void MainWindow::slotWriteSettings()
 {
     KronometerConfig::self()->save();
 
-    foreach (QWidget *widget, QApplication::topLevelWidgets()) {
+    foreach (auto widget, QApplication::topLevelWidgets()) {
         auto window = qobject_cast<MainWindow*>(widget);
 
         if (window) {
@@ -227,21 +229,21 @@ void MainWindow::slotUpdateLapDock()
 {
     m_lapView->resizeColumnsToContents();
     m_lapView->horizontalHeader()->setStretchLastSection(true);
-    m_lapView->selectRow(m_lapModel->rowCount(QModelIndex()) - 1);  // rows indexes start from 0
+    m_lapView->selectRow(m_lapModel->rowCount({}) - 1);  // rows indexes start from 0
 }
 
 void MainWindow::slotNewSession()
 {
-    auto window = new MainWindow();
+    auto window = new MainWindow {};
     window->show();
 }
 
 void MainWindow::slotOpenSession()
 {
-    QPointer<SessionDialog> dialog = new SessionDialog(nullptr, i18n("Sessions"));
+    QPointer<SessionDialog> dialog = new SessionDialog {nullptr, i18nc("@title:window", "Open Session")};
 
     if (dialog.data()->exec() == QDialog::Accepted) {
-        auto window = new MainWindow(nullptr, dialog.data()->selectedSession());
+        auto window = new MainWindow {nullptr, dialog.data()->selectedSession()};
         window->show();
     }
 
@@ -256,7 +258,7 @@ void MainWindow::slotSaveSession()
     m_session.clear();    // required for laps consistency
     m_session.setTime(m_stopwatch->raw());
 
-    for (int i = 0; i < m_lapModel->rowCount(QModelIndex()); i++) {
+    for (int i = 0; i < m_lapModel->rowCount({}); i++) {
         m_session.addLap(m_lapModel->at(i));
     }
 
@@ -268,7 +270,16 @@ void MainWindow::slotSaveSession()
 
 void MainWindow::slotSaveSessionAs()
 {
-    auto name = QInputDialog::getText(this, i18n("Choose a name"), i18n("Session name:"));
+    auto accepted = false;
+    auto name = QInputDialog::getText(this,
+                                      i18nc("@title:window", "Save session"),
+                                      i18n("You can choose a name for this session:"),
+                                      QLineEdit::Normal,
+                                      {},
+                                      &accepted);
+
+    if (not accepted)
+        return;
 
     if (name.isEmpty())
         name = i18nc("untitled session", "Untitled session");
@@ -278,10 +289,10 @@ void MainWindow::slotSaveSessionAs()
 
 void MainWindow::slotExportLapsAs()
 {
-    QPointer<QFileDialog> dialog = new QFileDialog(this);
+    QPointer<QFileDialog> dialog = new QFileDialog {this};
     dialog->setAcceptMode(QFileDialog::AcceptSave);
     dialog->setConfirmOverwrite(true);
-    dialog->setWindowTitle(i18n("Choose export file destination"));
+    dialog->setWindowTitle(i18nc("@title:window", "Export laps"));
 
     auto mimeTypes = { QStringLiteral("text/csv"), QStringLiteral("application/json") };
     dialog->setMimeTypeFilters(mimeTypes);
@@ -298,15 +309,103 @@ void MainWindow::slotCopyToClipboard()
     QApplication::clipboard()->setText(m_stopwatchDisplay->currentTime());
 }
 
+void MainWindow::slotToggleMenuBar()
+{
+    // Menubar shown for the first time.
+    if (KronometerConfig::menuBarNeverShown()) {
+        KronometerConfig::setMenuBarNeverShown(false);
+        slotWriteSettings();
+    }
+
+    menuBar()->setVisible(!menuBar()->isVisible());
+    m_toggleMenuAction->setChecked(menuBar()->isVisible());
+
+    menuBar()->isVisible() ? deleteControlMenuButton() : createControlMenuButton();
+}
+
+void MainWindow::slotUpdateControlMenu()
+{
+    auto menu = qobject_cast<QMenu*>(sender());
+    if (not menu)
+        return;
+
+    // All actions get cleared by QMenu::clear().
+    // This includes the sub-menus because 'menu' is their parent.
+    menu->clear();
+
+    auto ac = actionCollection();
+
+    // Add "File" actions
+    auto added = addActionToMenu(ac->action(QString::fromLatin1(KStandardAction::name(KStandardAction::New))), menu) |
+                 addActionToMenu(ac->action(QString::fromLatin1(KStandardAction::name(KStandardAction::Open))), menu);
+
+    if (added)
+        menu->addSeparator();
+
+    added = addActionToMenu(ac->action(QString::fromLatin1(KStandardAction::name(KStandardAction::Save))), menu) |
+            addActionToMenu(ac->action(QString::fromLatin1(KStandardAction::name(KStandardAction::SaveAs))), menu);
+
+    if (added)
+        menu->addSeparator();
+
+    added = addActionToMenu(m_exportAction, menu);
+
+    if (added)
+        menu->addSeparator();
+
+    // Add "Edit actions
+    added = addActionToMenu(ac->action(QString::fromLatin1(KStandardAction::name(KStandardAction::Copy))), menu);
+
+    if (added)
+        menu->addSeparator();
+
+    // Add "Settings" menu entries
+    addActionToMenu(ac->action(QString::fromLatin1(KStandardAction::name(KStandardAction::KeyBindings))), menu);
+    addActionToMenu(ac->action(QString::fromLatin1(KStandardAction::name(KStandardAction::ConfigureToolbars))), menu);
+    addActionToMenu(ac->action(QString::fromLatin1(KStandardAction::name(KStandardAction::Preferences))), menu);
+
+    // Add "Help" menu
+    auto helpMenu = new QMenu {i18nc("@action:inmenu", "Help"), menu};
+    helpMenu->addAction(ac->action(QString::fromLatin1(KStandardAction::name(KStandardAction::HelpContents))));
+    helpMenu->addAction(ac->action(QString::fromLatin1(KStandardAction::name(KStandardAction::WhatsThis))));
+    helpMenu->addSeparator();
+    helpMenu->addAction(ac->action(QString::fromLatin1(KStandardAction::name(KStandardAction::ReportBug))));
+    helpMenu->addSeparator();
+
+    // This action may be null, so must be checked before adding it to the help menu.
+    addActionToMenu(ac->action(QString::fromLatin1(KStandardAction::name(KStandardAction::SwitchApplicationLanguage))), helpMenu);
+
+    helpMenu->addSeparator();
+    helpMenu->addAction(ac->action(QString::fromLatin1(KStandardAction::name(KStandardAction::AboutApp))));
+    helpMenu->addAction(ac->action(QString::fromLatin1(KStandardAction::name(KStandardAction::AboutKDE))));
+    menu->addMenu(helpMenu);
+
+    menu->addSeparator();
+    addActionToMenu(ac->action(QString::fromLatin1(KStandardAction::name(KStandardAction::ShowMenubar))), menu);
+}
+
+void MainWindow::slotControlMenuButtonDeleted()
+{
+    m_controlMenuButton = nullptr;
+    m_controlMenuTimer->start();
+}
+
+void MainWindow::slotToolBarUpdated()
+{
+    if (!menuBar()->isVisible()) {
+        createControlMenuButton();
+    }
+}
+
 void MainWindow::setupCentralWidget()
 {
-    auto splitter = new QSplitter(this);
+    auto splitter = new QSplitter {this};
 
-    m_lapModel = new LapModel(this);
-    auto proxyModel = new QSortFilterProxyModel(this);
+    m_lapModel = new LapModel {this};
+    auto proxyModel = new QSortFilterProxyModel {this};
     proxyModel->setSourceModel(m_lapModel);
 
-    m_lapView = new QTableView(this);
+    m_lapView = new QTableView {this};
     m_lapView->setModel(proxyModel);
     m_lapView->setSelectionBehavior(QAbstractItemView::SelectRows);
     m_lapView->setGridStyle(Qt::DotLine);
@@ -324,34 +423,26 @@ void MainWindow::setupCentralWidget()
     setCentralWidget(splitter);
 }
 
-void MainWindow::setupStatusBar()
-{
-    m_statusLabel = new QLabel(this);
-    m_statusLabel->setToolTip(i18n("Current chronometer status"));
-
-    statusBar()->addWidget(m_statusLabel);
-}
-
 void MainWindow::setupActions()
 {
-    m_startAction = new QAction(this);
-    m_pauseAction = new QAction(this);
-    m_resetAction = new QAction(this);
-    m_lapAction = new QAction(this);
-    m_exportAction = new QAction(this);
+    m_startAction = new QAction {this};
+    m_pauseAction = new QAction {this};
+    m_resetAction = new QAction {this};
+    m_lapAction = new QAction {this};
+    m_exportAction = new QAction {this};
 
     m_startAction->setIcon(QIcon::fromTheme(QStringLiteral("chronometer-start")));
 
-    m_pauseAction->setText(i18n("&Pause"));  // pauseAction/resetAction have fixed text (startAction doesn't)
+    m_pauseAction->setText(i18nc("@action", "&Pause"));  // pauseAction/resetAction have fixed text (startAction doesn't)
     m_pauseAction->setIcon(QIcon::fromTheme(QStringLiteral("chronometer-pause")));
 
-    m_resetAction->setText(i18n("&Reset"));
+    m_resetAction->setText(i18nc("@action", "&Reset"));
     m_resetAction->setIcon(QIcon::fromTheme(QStringLiteral("chronometer-reset")));
 
-    m_lapAction->setText(i18n("&Lap"));
+    m_lapAction->setText(i18nc("@action", "&Lap"));
     m_lapAction->setIcon(QIcon::fromTheme(QStringLiteral("chronometer-lap")));
 
-    m_exportAction->setText(i18n("&Export laps as..."));
+    m_exportAction->setText(i18nc("@action:inmenu", "&Export Laps as..."));
     m_exportAction->setIcon(QIcon::fromTheme(QStringLiteral("document-export")));
 
     actionCollection()->addAction(QStringLiteral("start"), m_startAction);
@@ -392,6 +483,10 @@ void MainWindow::setupActions()
     KStandardAction::open(this, SLOT(slotOpenSession()), actionCollection());
     KStandardAction::copy(this, SLOT(slotCopyToClipboard()), actionCollection());
     connect(m_exportAction, &QAction::triggered, this, &MainWindow::slotExportLapsAs);
+
+    m_toggleMenuAction = KStandardAction::showMenubar(nullptr, nullptr, actionCollection());
+    // QueuedConnection prevents crashes when toggling the visibility
+    connect(m_toggleMenuAction, &KToggleAction::triggered, this, &MainWindow::slotToggleMenuBar, Qt::QueuedConnection);
 }
 
 void MainWindow::loadSettings()
@@ -399,8 +494,8 @@ void MainWindow::loadSettings()
     auto timeFrac = KronometerConfig::showSecondFractions() ? KronometerConfig::fractionsType() : TimeFormat::NoFractions;
     auto lapFrac = KronometerConfig::showSecondFractions() ? KronometerConfig::lapFractionsType() : TimeFormat::NoFractions;
 
-    TimeFormat timeFormat {KronometerConfig::showHours(), KronometerConfig::showMinutes(), timeFrac};
-    TimeFormat lapTimeFormat {KronometerConfig::showLapHours(), KronometerConfig::showLapMinutes(), lapFrac};
+    auto timeFormat = TimeFormat {KronometerConfig::showHours(), KronometerConfig::showMinutes(), timeFrac};
+    auto lapTimeFormat = TimeFormat {KronometerConfig::showLapHours(), KronometerConfig::showLapMinutes(), lapFrac};
 
     m_lapAction->setVisible(KronometerConfig::isLapsRecordingEnabled());
     m_exportAction->setVisible(KronometerConfig::isLapsRecordingEnabled());
@@ -416,9 +511,18 @@ void MainWindow::loadSettings()
     m_stopwatchDisplay->setFractionsFont(KronometerConfig::fracFont());
     m_stopwatchDisplay->setBackgroundColor(KronometerConfig::backgroundColor());
     m_stopwatchDisplay->setTextColor(KronometerConfig::textColor());
-    m_stopwatchDisplay->showHeaders(KronometerConfig::showTimeHeaders());
 
     setupGranularity();
+
+    // Always hide the menubar until the user shows it for the first time.
+    if (KronometerConfig::menuBarNeverShown()) {
+        menuBar()->hide();
+    }
+
+    if (menuBar()->isHidden()) {
+        m_toggleMenuAction->setChecked(false);
+        createControlMenuButton();
+    }
 }
 
 void MainWindow::setupGranularity()
@@ -445,10 +549,10 @@ void MainWindow::setupGranularity()
 
 void MainWindow::slotSaveSessionAs(const QString& name)
 {
-    Session newSession {m_stopwatch->raw()};
+    auto newSession = Session {m_stopwatch->raw()};
     newSession.setName(name);
 
-    for (int i = 0; i < m_lapModel->rowCount(QModelIndex()); i++) {
+    for (int i = 0; i < m_lapModel->rowCount({}); i++) {
         newSession.addLap(m_lapModel->at(i));
     }
 
@@ -464,7 +568,7 @@ void MainWindow::loadSession()
 {
     m_stopwatch->initialize(m_session.time());
 
-    foreach (const Lap& lap, m_session.laps()) {
+    foreach (const auto& lap, m_session.laps()) {
         m_lapModel->append(lap);
     }
 
@@ -491,7 +595,7 @@ void MainWindow::exportLapsAs(const QString& name, const QString& nameFilter)
         QJsonObject json;
         exportLapsAsJson(json);
 
-        QJsonDocument exportDoc {json};
+        auto exportDoc = QJsonDocument {json};
         exportFile.write(exportDoc.toJson());
         exportFile.commit();
     }
@@ -512,10 +616,9 @@ void MainWindow::exportLapsAs(const QString& name, const QString& nameFilter)
 
 void MainWindow::exportLapsAsJson(QJsonObject& json)
 {
-    QJsonArray laps;
-
-    for (int i = 0; i < m_lapModel->rowCount(QModelIndex()); i++) {
-        QJsonObject object;
+    auto laps = QJsonArray {};
+    for (auto i = 0; i < m_lapModel->rowCount({}); i++) {
+        auto object = QJsonObject {};
         m_lapModel->at(i).write(object);
         laps.append(object);
     }
@@ -526,9 +629,9 @@ void MainWindow::exportLapsAsJson(QJsonObject& json)
 void MainWindow::exportLapsAsCsv(QTextStream& out)
 {
     out << '#' << timestampMessage() << '\r' << '\n';
-    out << '#' << i18n("Lap number,Lap time,Global time,Note") << '\r' << '\n';
+    out << '#' << i18nc("@info:shell", "Lap number,Lap time,Global time,Note") << '\r' << '\n';
 
-    for (int i = 0; i < m_lapModel->rowCount(QModelIndex()); i++) {
+    for (auto i = 0; i < m_lapModel->rowCount({}); i++) {
         out << i;
         out << ',' << m_lapModel->at(i).relativeTime();
         out << ',' << m_lapModel->at(i).absoluteTime();
@@ -539,8 +642,64 @@ void MainWindow::exportLapsAsCsv(QTextStream& out)
 
 QString MainWindow::timestampMessage()
 {
-    auto timestamp = QDateTime::currentDateTime();
+    const auto timestamp = QDateTime::currentDateTime();
 
-    return i18n("Created by Kronometer on %1", timestamp.toString(Qt::DefaultLocaleLongDate));
+    return i18nc("@info:shell", "Created by Kronometer on %1", timestamp.toString(Qt::DefaultLocaleLongDate));
+}
+
+void MainWindow::createControlMenuButton()
+{
+    if (m_controlMenuButton) {
+        return;
+    }
+
+    m_controlMenuButton = new QToolButton {this};
+    m_controlMenuButton->setIcon(QIcon::fromTheme(QStringLiteral("application-menu")));
+    m_controlMenuButton->setPopupMode(QToolButton::InstantPopup);
+    m_controlMenuButton->setToolButtonStyle(toolBar()->toolButtonStyle());
+
+    auto controlMenu = new QMenu {m_controlMenuButton};
+    connect(controlMenu, &QMenu::aboutToShow, this, &MainWindow::slotUpdateControlMenu);
+
+    m_controlMenuButton->setMenu(controlMenu);
+
+    toolBar()->addWidget(m_controlMenuButton);
+    connect(toolBar(), &KToolBar::iconSizeChanged,
+            m_controlMenuButton, &QToolButton::setIconSize);
+    connect(toolBar(), &KToolBar::toolButtonStyleChanged,
+            m_controlMenuButton, &QToolButton::setToolButtonStyle);
+
+    // The control button may get deleted when e.g. the toolbar gets edited.
+    // In this case we must add it again. The adding is done asynchronously using a QTimer.
+    connect(m_controlMenuButton, &QToolButton::destroyed, this, &MainWindow::slotControlMenuButtonDeleted);
+    m_controlMenuTimer = new QTimer {this};
+    m_controlMenuTimer->setInterval(500);
+    m_controlMenuTimer->setSingleShot(true);
+    connect(m_controlMenuTimer, &QTimer::timeout, this, &MainWindow::slotToolBarUpdated);
+}
+
+void MainWindow::deleteControlMenuButton()
+{
+    delete m_controlMenuButton;
+    m_controlMenuButton = nullptr;
+
+    delete m_controlMenuTimer;
+    m_controlMenuTimer = nullptr;
+}
+
+bool MainWindow::addActionToMenu(QAction *action, QMenu *menu)
+{
+    if (not action or not menu)
+        return false;
+
+    const auto toolBarWidget = toolBar();
+    foreach (const auto widget, action->associatedWidgets()) {
+        if (widget == toolBarWidget) {
+            return false;
+        }
+    }
+
+    menu->addAction(action);
+    return true;
 }
 
