@@ -30,12 +30,15 @@ class SessionModel;
 class Stopwatch;
 class TimeDisplay;
 
+class KToggleAction;
+
 class QAction;
 class QLabel;
 class QSortFilterProxyModel;
 class QSplitter;
 class QTableView;
 class QTextStream;
+class QToolButton;
 
 /**
  * @brief Kronometer main window.
@@ -55,7 +58,7 @@ public:
 
 protected:
 
-    bool queryClose() Q_DECL_OVERRIDE;
+    bool queryClose() override;
 
 private slots:
 
@@ -125,18 +128,31 @@ private slots:
      */
     void slotCopyToClipboard();
 
+    /**
+     * Toggle menubar visibility.
+     */
+    void slotToggleMenuBar();
+
+    void slotUpdateControlMenu();
+
+    void slotControlMenuButtonDeleted();
+
+    void slotToolBarUpdated();
+
 private:
 
     Stopwatch *m_stopwatch;
     TimeDisplay *m_stopwatchDisplay;
     QTableView *m_lapView;
-    QLabel *m_statusLabel;
 
     QAction *m_startAction;
     QAction *m_pauseAction;
     QAction *m_resetAction;
     QAction *m_lapAction;
     QAction *m_exportAction;
+    QTimer *m_controlMenuTimer;
+    QToolButton *m_controlMenuButton;
+    KToggleAction *m_toggleMenuAction;
 
     LapModel *m_lapModel;
     SessionModel *m_sessionModel;
@@ -148,11 +164,6 @@ private:
      */
     void setupCentralWidget();
 
-    /**
-     * Setup application status bar.
-     */
-    void setupStatusBar();
-
     /**
      * Setup standard and custom QActions.
      */
@@ -204,6 +215,12 @@ private:
      */
     QString timestampMessage();
 
+    void createControlMenuButton();
+
+    void deleteControlMenuButton();
+
+    bool addActionToMenu(QAction* action, QMenu* menu);
+
     Q_DISABLE_COPY(MainWindow)
 };
 
