@@ -24,13 +24,16 @@
 
 class GeneralSettings : public QWidget, public Ui::GeneralSettings
 {
+    Q_OBJECT
 
 public:
-    explicit GeneralSettings(QWidget *parent) : QWidget(parent)
-    {
-        setupUi(this);
-    }
+    explicit GeneralSettings(QWidget *parent = nullptr);
+
+private slots:
+
+    void showFractionsToggled(bool toggled);
+    void showLapFractionsToggled(bool toggled);
+    void enableLapsToggled(bool toggled);
 };
 
-
 #endif
